#include <fstream>
#include <iostream>
#include <string>

#include <openbabel/mol.h>
#include <openbabel/obconversion.h>
#include <openbabel/forcefield.h>

using namespace std;
using namespace OpenBabel;

int main(int argc, char* argv[]) {
  // 力場の設定
  string ff = "MMFF94";
  OBForceField* pFF = OBForceField::FindForceField(ff);
  if (!pFF) {
    cerr << "指定の力場が見つかりませんでした" << endl;
    return -1;
  }
  pFF->SetLogFile(&cout);
  pFF->SetLogLevel(OBFF_LOGLVL_LOW);

  // ファイルオープン
  ifstream ifs("Arachidonic_Acid.sdf");
  if (ifs.fail()) {
    cerr << "ファイルが見つかりませんでした" << endl;
    return -1;
  }

  // 参照分子の読み込み
  OBConversion conv;
  conv.SetInAndOutFormats("SDF", "SDF");
  OBMol mol;
  conv.Read(&mol, &ifs);
  ifs.close();

  mol.AddHydrogens();  // 水素付加

  // 分子をセットアップ（部分電荷などの計算）
  if (!pFF->Setup(mol)) {
    cerr << "分子の設定に失敗しました" << endl;
    return -1;
  }

  // 最急降下法からの共役勾配法でエネルギー極小化
  pFF->SteepestDescent(5000, 1e-1);
  pFF->GetCoordinates(mol);
  pFF->ConjugateGradients(5000);
  pFF->GetCoordinates(mol);

  // 分子を出力
  ofstream ofs("Out_Arachidonic_Acid.sdf");
  conv.Write(&mol, &ofs);
  ofs.close();

  return 0;
}
